@@ -65,11 +65,6 @@
 #include "euidindex.h"
 #include "context.h"
 #include "svn_revision.h"
-
-#ifndef HAVE_SNPRINTF
-#include "snprintf.h"
-#endif
-
 #include "ctdl_module.h"
 
 char *unique_session_numbers;
@@ -216,7 +211,7 @@ void master_cleanup(int exitcode) {
 	struct CleanupFunctionHook *fcn;
 	static int already_cleaning_up = 0;
 
-	if (already_cleaning_up) while(1) usleep(1000);
+	if (already_cleaning_up) while(1) usleep(1000000);
 	already_cleaning_up = 1;
 
 	/* Run any cleanup routines registered by loadable modules */
@@ -427,7 +422,7 @@ int is_public_client(void)
 					}
 				}
 			}
-		fclose(fp);
+		if (fp != NULL) fclose(fp);
 		pc_timestamp = time(NULL);
 		end_critical_section(S_PUBLIC_CLIENTS);
 	}
@@ -686,14 +681,14 @@ int CtdlAccessCheck(int required_level) {
 
 	if (CC->user.axlevel >= AxAideU) return(0);
  	if (required_level >= ac_aide) {
-		cprintf("%d This command requires Aide access.\n",
+		cprintf("%d This command requires Admin access.\n",
 			ERROR + HIGHER_ACCESS_REQUIRED);
 		return(-1);
 	}
 
 	if (is_room_aide()) return(0);
 	if (required_level >= ac_room_aide) {
-		cprintf("%d This command requires Aide or Room Aide access.\n",
+		cprintf("%d This command requires Admin or Room Admin access.\n",
 			ERROR + HIGHER_ACCESS_REQUIRED);
 		return(-1);
 	}
