@@ -12,14 +12,17 @@
 
 #include "serv_extensions.h"
 #include "context.h"
+#include "ctdl_module.h"
 
 /* Simple linked list structures ... used in a bunch of different places. */
 typedef struct RoomProcList RoomProcList;
 struct RoomProcList {
         struct RoomProcList *next;
+	OneRoomNetCfg *OneRNCfg;
         char name[ROOMNAMELEN];
         char lcname[ROOMNAMELEN];
 	long namelen;
+	long lastsent;
 	long key;
 	long QRNum;
 };
