@@ -931,7 +931,7 @@ time_t CheckIfAlreadySeen(const char *Facility,
 	{
 		SEENM_syslog(LOG_DEBUG, "Loading");
 		cdbut = cdb_fetch(CDB_USETABLE, SKEY(guid));
-		if (cdbut != NULL) {
+		if ((cdbut != NULL) && (cdbut->ptr != NULL)) {
 			memcpy(&ut, cdbut->ptr,
 			       ((cdbut->len > sizeof(struct UseTable)) ?
 				sizeof(struct UseTable) : cdbut->len));
@@ -952,6 +952,8 @@ time_t CheckIfAlreadySeen(const char *Facility,
 		}
 		else
 		{
+			if (cdbut) cdb_free(cdbut);
+
 			SEENM_syslog(LOG_DEBUG, "not Found");
 		}
 
