@@ -64,6 +64,49 @@
 #include "event_client.h"
 #include "ctdl_module.h"
 
+
+ConstStr IOStates[] = {
+	{HKEY("DB Queue")},
+	{HKEY("DB Q Next")},
+	{HKEY("DB Attach")},
+	{HKEY("DB Next")},
+	{HKEY("DB Stop")},
+	{HKEY("DB Exit")},
+	{HKEY("DB Terminate")},
+	{HKEY("IO Queue")},
+	{HKEY("IO Attach")},
+	{HKEY("IO Connect Socket")},
+	{HKEY("IO Abort")},
+	{HKEY("IO Timeout")},
+	{HKEY("IO ConnFail")},
+	{HKEY("IO ConnFail Now")},
+	{HKEY("IO Conn Now")},
+	{HKEY("IO Conn Wait")},
+	{HKEY("Curl Q")},
+	{HKEY("Curl Start")},
+	{HKEY("Curl Shotdown")},
+	{HKEY("Curl More IO")},
+	{HKEY("Curl Got IO")},
+	{HKEY("Curl Got Data")},
+	{HKEY("Curl Got Status")},
+	{HKEY("C-Ares Start")},
+	{HKEY("C-Ares IO Done")},
+	{HKEY("C-Ares Finished")},
+	{HKEY("C-Ares exit")},
+	{HKEY("Killing")},
+	{HKEY("Exit")}
+};
+
+void SetEVState(AsyncIO *IO, eIOState State)
+{
+
+	CitContext* CCC = IO->CitContext;
+	if (CCC != NULL)
+		memcpy(CCC->lastcmdname, IOStates[State].Key, IOStates[State].len + 1);
+
+}
+
+
 static void IO_Timeout_callback(struct ev_loop *loop, ev_timer *watcher, int revents);
 static void IO_abort_shutdown_callback(struct ev_loop *loop,
 				       ev_cleanup *watcher,
@@ -86,6 +129,7 @@ eNextState QueueDBOperation(AsyncIO *IO, IO_CallBack CB)
 	IOAddHandler *h;
 	int i;
 
+	SetEVState(IO, eDBQ);
 	h = (IOAddHandler*)malloc(sizeof(IOAddHandler));
 	h->IO = IO;
 	h->EvAttch = CB;
@@ -122,6 +166,7 @@ eNextState QueueDBOperation(AsyncIO *IO, IO_CallBack CB)
 
 void StopDBWatchers(AsyncIO *IO)
 {
+	SetEVState(IO, eDBStop);
 	ev_cleanup_stop(event_db, &IO->db_abort_by_shutdown);
 	ev_idle_stop(event_db, &IO->db_unwind_stack);
 }
@@ -131,6 +176,7 @@ void ShutDownDBCLient(AsyncIO *IO)
 	CitContext *Ctx =IO->CitContext;
 	become_session(Ctx);
 
+	SetEVState(IO, eDBTerm);
 	EVM_syslog(LOG_DEBUG, "DBEVENT Terminating.\n");
 	StopDBWatchers(IO);
 
@@ -142,6 +188,8 @@ void
 DB_PerformNext(struct ev_loop *loop, ev_idle *watcher, int revents)
 {
 	AsyncIO *IO = watcher->data;
+
+	SetEVState(IO, eDBNext);
 	IO->Now = ev_now(event_db);
 	EV_syslog(LOG_DEBUG, "%s()", __FUNCTION__);
 	become_session(IO->CitContext);
@@ -175,6 +223,7 @@ DB_PerformNext(struct ev_loop *loop, ev_idle *watcher, int revents)
 
 eNextState NextDBOperation(AsyncIO *IO, IO_CallBack CB)
 {
+	SetEVState(IO, eQDBNext);
 	IO->NextDBOperation = CB;
 	ev_idle_init(&IO->db_unwind_stack,
 		     DB_PerformNext);
@@ -199,6 +248,8 @@ static void IO_abort_shutdown_callback(struct ev_loop *loop,
 				       int revents)
 {
 	AsyncIO *IO = watcher->data;
+
+	SetEVState(IO, eIOAbort);
 	EV_syslog(LOG_DEBUG, "EVENT Q: %s\n", __FUNCTION__);
 	IO->Now = ev_now(event_base);
 	assert(IO->ShutdownAbort);
@@ -211,6 +262,7 @@ eNextState QueueEventContext(AsyncIO *IO, IO_CallBack CB)
 	IOAddHandler *h;
 	int i;
 
+	SetEVState(IO, eIOQ);
 	h = (IOAddHandler*)malloc(sizeof(IOAddHandler));
 	h->IO = IO;
 	h->EvAttch = CB;
@@ -243,6 +295,17 @@ eNextState QueueEventContext(AsyncIO *IO, IO_CallBack CB)
 	return eSendReply;
 }
 
+eNextState EventQueueDBOperation(AsyncIO *IO, IO_CallBack CB)
+{
+	StopClientWatchers(IO, 0);
+	return QueueDBOperation(IO, CB);
+}
+eNextState DBQueueEventContext(AsyncIO *IO, IO_CallBack CB)
+{
+	StopDBWatchers(IO);
+	return QueueEventContext(IO, CB);
+}
+
 extern eNextState evcurl_handle_start(AsyncIO *IO);
 
 eNextState QueueCurlContext(AsyncIO *IO)
@@ -250,6 +313,7 @@ eNextState QueueCurlContext(AsyncIO *IO)
 	IOAddHandler *h;
 	int i;
 
+	SetEVState(IO, eCurlQ);
 	h = (IOAddHandler*)malloc(sizeof(IOAddHandler));
 	h->IO = IO;
 	h->EvAttch = evcurl_handle_start;
@@ -281,6 +345,13 @@ eNextState QueueCurlContext(AsyncIO *IO)
 	return eSendReply;
 }
 
+eNextState CurlQueueDBOperation(AsyncIO *IO, IO_CallBack CB)
+{
+	StopCurlWatchers(IO);
+	return QueueDBOperation(IO, CB);
+}
+
+
 void DestructCAres(AsyncIO *IO);
 void FreeAsyncIOContents(AsyncIO *IO)
 {
@@ -298,6 +369,7 @@ void FreeAsyncIOContents(AsyncIO *IO)
 	if (Ctx) {
 		Ctx->state = CON_IDLE;
 		Ctx->kill_me = 1;
+		IO->CitContext = NULL;
 	}
 }
 
@@ -315,13 +387,15 @@ void StopClientWatchers(AsyncIO *IO, int CloseFD)
 
 	if (CloseFD && (IO->SendBuf.fd > 0)) {
 		close(IO->SendBuf.fd);
-		IO->SendBuf.fd = -1;
-		IO->RecvBuf.fd = -1;
+		IO->SendBuf.fd = 0;
+		IO->RecvBuf.fd = 0;
 	}
 }
 
 void StopCurlWatchers(AsyncIO *IO)
 {
+	EVM_syslog(LOG_DEBUG, "EVENT StopCurlWatchers \n");
+
 	ev_timer_stop (event_base, &IO->rw_timeout);
 	ev_timer_stop(event_base, &IO->conn_fail);
 	ev_idle_stop(event_base, &IO->unwind_stack);
@@ -331,6 +405,9 @@ void StopCurlWatchers(AsyncIO *IO)
 	ev_io_stop(event_base, &IO->send_event);
 	ev_io_stop(event_base, &IO->recv_event);
 
+	curl_easy_cleanup(IO->HttpReq.chnd);
+	IO->HttpReq.chnd = NULL;
+
 	if (IO->SendBuf.fd != 0) {
 		close(IO->SendBuf.fd);
 	}
@@ -341,6 +418,8 @@ void StopCurlWatchers(AsyncIO *IO)
 void ShutDownCLient(AsyncIO *IO)
 {
 	CitContext *Ctx =IO->CitContext;
+
+	SetEVState(IO, eExit);
 	become_session(Ctx);
 
 	EVM_syslog(LOG_DEBUG, "EVENT Terminating \n");
@@ -595,12 +674,12 @@ IO_send_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 		if (errno != EAGAIN) {
 			StopClientWatchers(IO, 1);
 			EV_syslog(LOG_DEBUG,
-				  "EVENT: Socket Invalid! [%d] [%s] [%d]\n",
+				  "IO_send_callback(): Socket Invalid! [%d] [%s] [%d]\n",
 				  errno, strerror(errno), IO->SendBuf.fd);
 			StrBufPrintf(IO->ErrMsg,
 				     "Socket Invalid! [%s]",
 				     strerror(errno));
-			SetNextTimeout(IO, 0.0);
+			SetNextTimeout(IO, 0.01);
 		}
 	}
 	/* else : must write more. */
@@ -641,6 +720,7 @@ IO_Timeout_callback(struct ev_loop *loop, ev_timer *watcher, int revents)
 {
 	AsyncIO *IO = watcher->data;
 
+	SetEVState(IO, eIOTimeout);
 	IO->Now = ev_now(event_base);
 	ev_timer_stop (event_base, &IO->rw_timeout);
 	become_session(IO->CitContext);
@@ -669,6 +749,7 @@ IO_connfail_callback(struct ev_loop *loop, ev_timer *watcher, int revents)
 {
 	AsyncIO *IO = watcher->data;
 
+	SetEVState(IO, eIOConnfail);
 	IO->Now = ev_now(event_base);
 	ev_timer_stop (event_base, &IO->conn_fail);
 
@@ -701,6 +782,7 @@ IO_connfailimmediate_callback(struct ev_loop *loop,
 {
 	AsyncIO *IO = watcher->data;
 
+	SetEVState(IO, eIOConnfailNow);
 	IO->Now = ev_now(event_base);
 	ev_idle_stop (event_base, &IO->conn_fail_immediate);
 
@@ -730,6 +812,7 @@ IO_connestd_callback(struct ev_loop *loop, ev_io *watcher, int revents)
         socklen_t       lon = sizeof(so_err);
         int             err;
 
+	SetEVState(IO, eIOConnNow);
         IO->Now = ev_now(event_base);
         EVM_syslog(LOG_DEBUG, "connect() succeeded.\n");
 
@@ -828,12 +911,12 @@ IO_recv_callback(struct ev_loop *loop, ev_io *watcher, int revents)
 			// FD is gone. kick it. 
 			StopClientWatchers(IO, 1);
 			EV_syslog(LOG_DEBUG,
-				  "EVENT: Socket Invalid! [%d] [%s] [%d]\n",
+				  "IO_recv_callback(): Socket Invalid! [%d] [%s] [%d]\n",
 				  errno, strerror(errno), IO->SendBuf.fd);
 			StrBufPrintf(IO->ErrMsg,
 				     "Socket Invalid! [%s]",
 				     strerror(errno));
-			SetNextTimeout(IO, 0.0);
+			SetNextTimeout(IO, 0.01);
 		}
 		return;
 	}
@@ -843,6 +926,8 @@ void
 IO_postdns_callback(struct ev_loop *loop, ev_idle *watcher, int revents)
 {
 	AsyncIO *IO = watcher->data;
+
+	SetEVState(IO, eCaresFinished);
 	IO->Now = ev_now(event_base);
 	EV_syslog(LOG_DEBUG, "event: %s\n", __FUNCTION__);
 	become_session(IO->CitContext);
@@ -873,6 +958,7 @@ eNextState EvConnectSock(AsyncIO *IO,
 	int fdflags;
 	int rc = -1;
 
+	SetEVState(IO, eIOConnectSock);
 	become_session(IO->CitContext);
 
 	if (ReadFirst) {
@@ -902,10 +988,12 @@ eNextState EvConnectSock(AsyncIO *IO,
 	fdflags = fcntl(IO->SendBuf.fd, F_GETFL);
 	if (fdflags < 0) {
 		EV_syslog(LOG_ERR,
-			  "EVENT: unable to get socket flags! %s \n",
+			  "EVENT: unable to get socket %d flags! %s \n",
+			  IO->SendBuf.fd,
 			  strerror(errno));
 		StrBufPrintf(IO->ErrMsg,
-			     "Failed to get socket flags: %s",
+			     "Failed to get socket %d flags: %s",
+			     IO->SendBuf.fd,
 			     strerror(errno));
 		close(IO->SendBuf.fd);
 		IO->SendBuf.fd = IO->RecvBuf.fd = 0;
@@ -915,7 +1003,8 @@ eNextState EvConnectSock(AsyncIO *IO,
 	if (fcntl(IO->SendBuf.fd, F_SETFL, fdflags) < 0) {
 		EV_syslog(
 			LOG_ERR,
-			"EVENT: unable to set socket nonblocking flags! %s \n",
+			"EVENT: unable to set socket %d nonblocking flags! %s \n",
+			IO->SendBuf.fd,
 			strerror(errno));
 		StrBufPrintf(IO->ErrMsg,
 			     "Failed to set socket flags: %s",
@@ -972,12 +1061,14 @@ eNextState EvConnectSock(AsyncIO *IO,
 	}
 
 	if (rc >= 0){
-		EVM_syslog(LOG_DEBUG, "connect() immediate success.\n");
+		SetEVState(IO, eIOConnNow);
+		EV_syslog(LOG_DEBUG, "connect() = %d immediate success.\n", IO->SendBuf.fd);
 		set_start_callback(event_base, IO, 0);
 		return IO->NextState;
 	}
 	else if (errno == EINPROGRESS) {
-		EVM_syslog(LOG_DEBUG, "connect() have to wait now.\n");
+		SetEVState(IO, eIOConnWait);
+		EV_syslog(LOG_DEBUG, "connect() = %d have to wait now.\n", IO->SendBuf.fd);
 
 		ev_io_init(&IO->conn_event,
 			   IO_connestd_callback,
@@ -991,12 +1082,17 @@ eNextState EvConnectSock(AsyncIO *IO,
 		return IO->NextState;
 	}
 	else {
+		SetEVState(IO, eIOConnfail);
 		ev_idle_init(&IO->conn_fail_immediate,
 			     IO_connfailimmediate_callback);
 		IO->conn_fail_immediate.data = IO;
 		ev_idle_start(event_base, &IO->conn_fail_immediate);
 
-		EV_syslog(LOG_ERR, "connect() failed: %s\n", strerror(errno));
+		EV_syslog(LOG_ERR,
+			  "connect() = %d failed: %s\n",
+			  IO->SendBuf.fd,
+			  strerror(errno));
+
 		StrBufPrintf(IO->ErrMsg,
 			     "Failed to connect: %s",
 			     strerror(errno));
@@ -1016,6 +1112,7 @@ eNextState ReAttachIO(AsyncIO *IO,
 		      void *pData,
 		      int ReadFirst)
 {
+	SetEVState(IO, eIOAttach);
 	IO->Data = pData;
 	become_session(IO->CitContext);
 	ev_cleanup_start(event_base, &IO->abort_by_shutdown);
@@ -1046,7 +1143,8 @@ void InitIOStruct(AsyncIO *IO,
 	IO->Data          = Data;
 
 	IO->CitContext    = CloneContext(CC);
-	((CitContext *)IO->CitContext)->session_specific_data = (char*) Data;
+	IO->CitContext->session_specific_data = Data;
+	IO->CitContext->IO = IO;
 
 	IO->NextState     = NextState;
 
@@ -1083,7 +1181,8 @@ int InitcURLIOStruct(AsyncIO *IO,
 	IO->Data          = Data;
 
 	IO->CitContext    = CloneContext(CC);
-	((CitContext *)IO->CitContext)->session_specific_data = (char*) Data;
+	IO->CitContext->session_specific_data = Data;
+	IO->CitContext->IO = IO;
 
 	IO->SendDone      = SendDone;
 	IO->Terminate     = Terminate;
@@ -1097,6 +1196,90 @@ int InitcURLIOStruct(AsyncIO *IO,
 
 }
 
+
+typedef struct KillOtherSessionContext {
+	AsyncIO IO;
+	AsyncIO *OtherOne;
+}KillOtherSessionContext;
+
+eNextState KillTerminate(AsyncIO *IO)
+{
+	long id;
+	KillOtherSessionContext *Ctx = (KillOtherSessionContext*)IO->Data;
+	EV_syslog(LOG_DEBUG, "%s Exit\n", __FUNCTION__);
+	id = IO->ID;
+	FreeAsyncIOContents(IO);
+	memset(Ctx, 0, sizeof(KillOtherSessionContext));
+	IO->ID = id; /* just for the case we want to analyze it in a coredump */
+	free(Ctx);
+	return eAbort;
+
+}
+
+eNextState KillShutdown(AsyncIO *IO)
+{
+	return eTerminateConnection;
+}
+
+eNextState KillOtherContextNow(AsyncIO *IO)
+{
+	KillOtherSessionContext *Ctx = IO->Data;
+
+	SetEVState(IO, eKill);
+
+	if (Ctx->OtherOne->ShutdownAbort != NULL)
+		Ctx->OtherOne->ShutdownAbort(Ctx->OtherOne);
+	return eTerminateConnection;
+}
+
+void KillAsyncIOContext(AsyncIO *IO)
+{
+	KillOtherSessionContext *Ctx;
+
+	Ctx = (KillOtherSessionContext*) malloc(sizeof(KillOtherSessionContext));
+	memset(Ctx, 0, sizeof(KillOtherSessionContext));
+	
+	InitIOStruct(&Ctx->IO,
+		     Ctx,
+		     eReadMessage,
+		     NULL,
+		     NULL,
+		     NULL,
+		     NULL,
+		     KillTerminate,
+		     NULL,
+		     NULL,
+		     NULL,
+		     KillShutdown);
+
+	Ctx->OtherOne = IO;
+
+	switch(IO->NextState) {
+	case eSendDNSQuery:
+	case eReadDNSReply:
+
+	case eConnect:
+	case eSendReply:
+	case eSendMore:
+	case eSendFile:
+
+	case eReadMessage:
+	case eReadMore:
+	case eReadPayload:
+	case eReadFile:
+		QueueEventContext(&Ctx->IO, KillOtherContextNow);
+		break;
+	case eDBQuery:
+		QueueDBOperation(&Ctx->IO, KillOtherContextNow);
+		break;
+	case eTerminateConnection:
+	case eAbort:
+		/*hm, its already dying, dunno which Queue its in... */
+		free(Ctx);
+	}
+	
+}
+
 extern int DebugEventLoopBacktrace;
 void EV_backtrace(AsyncIO *IO)
 {
