@@ -113,7 +113,7 @@ void cmd_lbio(char *cmdbuf)
 	       (filedir_entry != NULL))
 	{
 #ifdef _DIRENT_HAVE_D_NAMLEN
-		d_namelen = filedir_entry->d_namelen;
+		d_namelen = filedir_entry->d_namlen;
 
 #else
 		d_namelen = strlen(filedir_entry->d_name);
