@@ -194,7 +194,7 @@ void FreeMailQEntry(void *qv)
 */
 	FreeStrBuf(&Q->Recipient);
 	FreeStrBuf(&Q->StatusMessage);
-
+	FreeStrBuf(&Q->AllStatusMessages);
 	memset(Q, 0, sizeof(MailQEntry));
 	free(Q);
 }
@@ -385,7 +385,10 @@ StrBuf *SerializeQueueItem(OneQueItem *MyQItem)
 		StrBufAppendBufPlain(QMessage, HKEY("|"), 0);
 		StrBufAppendPrintf(QMessage, "%d", ThisItem->Status);
 		StrBufAppendBufPlain(QMessage, HKEY("|"), 0);
-		StrBufAppendBuf(QMessage, ThisItem->StatusMessage, 0);
+		if (ThisItem->AllStatusMessages != NULL)
+			StrBufAppendBuf(QMessage, ThisItem->AllStatusMessages, 0);
+		else
+			StrBufAppendBuf(QMessage, ThisItem->StatusMessage, 0);
 	}
 	DeleteHashPos(&It);
 	StrBufAppendBufPlain(QMessage, HKEY("\n"), 0);
@@ -478,7 +481,7 @@ void QItem_Handle_Attempted(OneQueItem *Item, StrBuf *Line, const char **Pos)
 /**
  * this one has to have the context for loading the message via the redirect buffer...
  */
-StrBuf *smtp_load_msg(OneQueItem *MyQItem, int n)
+StrBuf *smtp_load_msg(OneQueItem *MyQItem, int n, char **Author, char **Address)
 {
 	CitContext *CCC=CC;
 	StrBuf *SendMsg;
@@ -487,7 +490,9 @@ StrBuf *smtp_load_msg(OneQueItem *MyQItem, int n)
 	CtdlOutputMsg(MyQItem->MessageID,
 		      MT_RFC822, HEADERS_ALL,
 		      0, 1, NULL,
-		      (ESC_DOT|SUPPRESS_ENV_TO) );
+		      (ESC_DOT|SUPPRESS_ENV_TO),
+		      Author,
+		      Address);
 
 	SendMsg = CCC->redirect_buffer;
 	CCC->redirect_buffer = NULL;
@@ -513,7 +518,7 @@ StrBuf *smtp_load_msg(OneQueItem *MyQItem, int n)
 void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt, ParsedURL *Relay)
 {
 	static int seq = 0;
-
+	
 	struct CtdlMessage *bmsg = NULL;
 	StrBuf *boundary;
 	StrBuf *Msg = NULL;
@@ -564,7 +569,10 @@ void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt, ParsedURL *Relay)
 			StrBufAppendBufPlain(Msg, HKEY(" "), 0);
 			StrBufAppendBuf(Msg, ThisItem->Recipient, 0);
 			StrBufAppendBufPlain(Msg, HKEY(": "), 0);
-			StrBufAppendBuf(Msg, ThisItem->StatusMessage, 0);
+			if (ThisItem->AllStatusMessages != NULL)
+				StrBufAppendBuf(Msg, ThisItem->AllStatusMessages, 0);
+			else
+				StrBufAppendBuf(Msg, ThisItem->StatusMessage, 0);
 			StrBufAppendBufPlain(Msg, HKEY("\r\n"), 0);
 		}
 	}
@@ -579,12 +587,16 @@ void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt, ParsedURL *Relay)
 	}
 
 	if ((StrLength(MyQItem->SenderRoom) == 0) && MyQItem->HaveRelay) {
+		const char *RelayUrlStr = "[not found]";
 		/* one message that relaying is broken is enough; no extra room error message. */
 		StrBuf *RelayDetails = NewStrBuf();
 
+		if (Relay != NULL)
+			RelayUrlStr = ChrPtr(Relay->URL);
+
 		StrBufPrintf(RelayDetails,
 			     "Relaying via %s failed permanently. \n Reason:\n%s\n Revalidate your relay configuration.",
-			     ChrPtr(Relay->URL),
+			     RelayUrlStr,
 			     ChrPtr(Msg));
                 CtdlAideMessage(ChrPtr(RelayDetails), "Relaying Failed");
 		FreeStrBuf(&RelayDetails);
@@ -722,6 +734,81 @@ void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt, ParsedURL *Relay)
 	SMTPCM_syslog(LOG_DEBUG, "Done processing bounces\n");
 }
 
+ParsedURL *LoadRelayUrls(OneQueItem *MyQItem,
+			 char *Author,
+			 char *Address)
+{
+	int nRelays = 0;
+	ParsedURL *RelayUrls = NULL;
+	char mxbuf[SIZ];
+	ParsedURL **Url = &MyQItem->URL;
+
+	nRelays = get_hosts(mxbuf, "fallbackhost");
+	if (nRelays > 0) {
+		StrBuf *All;
+		StrBuf *One;
+		const char *Pos = NULL;
+		All = NewStrBufPlain(mxbuf, -1);
+		One = NewStrBufPlain(NULL, StrLength(All) + 1);
+		
+		while ((Pos != StrBufNOTNULL) &&
+		       ((Pos == NULL) ||
+			!IsEmptyStr(Pos)))
+		{
+			StrBufExtract_NextToken(One, All, &Pos, '|');
+			if (!ParseURL(Url, One, DefaultMXPort)) {
+				SMTPC_syslog(LOG_DEBUG,
+					     "Failed to parse: %s\n",
+					     ChrPtr(One));
+			}
+			else {
+				(*Url)->IsRelay = 1;
+				MyQItem->HaveRelay = 1;
+			}
+		}
+		FreeStrBuf(&All);
+		FreeStrBuf(&One);
+	}
+	nRelays = get_hosts(mxbuf, "smarthost");
+	if (nRelays > 0) {
+		char *User;
+		StrBuf *All;
+		StrBuf *One;
+		const char *Pos = NULL;
+		All = NewStrBufPlain(mxbuf, -1);
+		One = NewStrBufPlain(NULL, StrLength(All) + 1);
+		
+		while ((Pos != StrBufNOTNULL) &&
+		       ((Pos == NULL) ||
+			!IsEmptyStr(Pos)))
+		{
+			StrBufExtract_NextToken(One, All, &Pos, '|');
+			User = strchr(ChrPtr(One), ' ');
+			if (User != NULL) {
+				if (!strcmp(User + 1, Author) ||
+				    !strcmp(User + 1, Address))
+					StrBufCutAt(One, 0, User);
+				else {
+					MyQItem->HaveRelay = 1;
+					continue;
+				}
+			}
+			if (!ParseURL(Url, One, DefaultMXPort)) {
+				SMTPC_syslog(LOG_DEBUG,
+					     "Failed to parse: %s\n",
+					     ChrPtr(One));
+			}
+			else {
+				///if (!Url->IsIP)) // todo dupe me fork ipv6
+				(*Url)->IsRelay = 1;
+				MyQItem->HaveRelay = 1;
+			}
+		}
+		FreeStrBuf(&All);
+		FreeStrBuf(&One);
+	}
+	return RelayUrls;
+}
 /*
  * smtp_do_procmsg()
  *
@@ -731,6 +818,8 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	time_t now;
 	int mynumsessions = num_sessions;
 	struct CtdlMessage *msg = NULL;
+	char *Author = NULL;
+	char *Address = NULL;
 	char *instr = NULL;
 	StrBuf *PlainQItem;
 	OneQueItem *MyQItem;
@@ -739,13 +828,11 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	void *vQE;
 	long len;
 	const char *Key;
-	int nRelays = 0;
-	ParsedURL *RelayUrls = NULL;
 	int HaveBuffers = 0;
 	StrBuf *Msg =NULL;
 
 	if (mynumsessions > max_sessions_for_outbound_smtp) {
-		SMTPC_syslog(LOG_DEBUG,
+		SMTPC_syslog(LOG_INFO,
 			     "skipping because of num jobs %d > %d max_sessions_for_outbound_smtp",
 			     mynumsessions,
 			     max_sessions_for_outbound_smtp);
@@ -832,64 +919,6 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 		return;
 	}
 
-	{
-		char mxbuf[SIZ];
-		ParsedURL **Url = &MyQItem->URL;
-		nRelays = get_hosts(mxbuf, "fallbackhost");
-		if (nRelays > 0) {
-			StrBuf *All;
-			StrBuf *One;
-			const char *Pos = NULL;
-			All = NewStrBufPlain(mxbuf, -1);
-			One = NewStrBufPlain(NULL, StrLength(All) + 1);
-
-			while ((Pos != StrBufNOTNULL) &&
-			       ((Pos == NULL) ||
-				!IsEmptyStr(Pos)))
-			{
-				StrBufExtract_NextToken(One, All, &Pos, '|');
-				if (!ParseURL(Url, One, DefaultMXPort)) {
-					SMTPC_syslog(LOG_DEBUG,
-						     "Failed to parse: %s\n",
-						     ChrPtr(One));
-				}
-				else {
-					(*Url)->IsRelay = 1;
-					MyQItem->HaveRelay = 1;
-				}
-			}
-			FreeStrBuf(&All);
-			FreeStrBuf(&One);
-		}
-		nRelays = get_hosts(mxbuf, "smarthost");
-		if (nRelays > 0) {
-			StrBuf *All;
-			StrBuf *One;
-			const char *Pos = NULL;
-			All = NewStrBufPlain(mxbuf, -1);
-			One = NewStrBufPlain(NULL, StrLength(All) + 1);
-
-			while ((Pos != StrBufNOTNULL) &&
-			       ((Pos == NULL) ||
-				!IsEmptyStr(Pos)))
-			{
-				StrBufExtract_NextToken(One, All, &Pos, '|');
-				if (!ParseURL(Url, One, DefaultMXPort)) {
-					SMTPC_syslog(LOG_DEBUG,
-						     "Failed to parse: %s\n",
-						     ChrPtr(One));
-				}
-				else {
-					///if (!Url->IsIP)) // todo dupe me fork ipv6
-					(*Url)->IsRelay = 1;
-					MyQItem->HaveRelay = 1;
-				}
-			}
-			FreeStrBuf(&All);
-			FreeStrBuf(&One);
-		}
-
-	}
 
 	It = GetNewHashPos(MyQItem->MailQEntries, 0);
 	while (GetNextHashPos(MyQItem->MailQEntries, It, &len, &Key, &vQE))
@@ -914,13 +943,23 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	    (((MyQItem->ActiveDeliveries * 2)  < max_sessions_for_outbound_smtp)))
 	{
 		/* abort delivery for another time. */
-		SMTPC_syslog(LOG_DEBUG,
+		SMTPC_syslog(LOG_INFO,
 			     "SMTP Queue: skipping because of num jobs %d + %ld > %d max_sessions_for_outbound_smtp",
 			     mynumsessions,
 			     MyQItem->ActiveDeliveries,
 			     max_sessions_for_outbound_smtp);
 
-		FreeQueItem(&MyQItem);
+		It = GetNewHashPos(MyQItem->MailQEntries, 0);
+		pthread_mutex_lock(&ActiveQItemsLock);
+		{
+			if (GetHashPosFromKey(ActiveQItems,
+					      LKEY(MyQItem->MessageID),
+					      It))
+			{
+				DeleteEntryFromHash(ActiveQItems, It);
+			}
+		}
+		pthread_mutex_unlock(&ActiveQItemsLock);
 
 		return;
 	}
@@ -928,12 +967,60 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 
 	if (MyQItem->ActiveDeliveries > 0)
 	{
+		ParsedURL *RelayUrls = NULL;
 		int nActivated = 0;
 		int n = MsgCount++;
 		int m = MyQItem->ActiveDeliveries;
 		int i = 1;
-		Msg = smtp_load_msg(MyQItem, n);
+
 		It = GetNewHashPos(MyQItem->MailQEntries, 0);
+
+		Msg = smtp_load_msg(MyQItem, n, &Author, &Address);
+		RelayUrls = LoadRelayUrls(MyQItem, Author, Address);
+		if ((RelayUrls == NULL) && MyQItem->HaveRelay) {
+
+			while ((i <= m) &&
+			       (GetNextHashPos(MyQItem->MailQEntries,
+					       It, &len, &Key, &vQE)))
+			{
+				int KeepBuffers = (i == m);
+				MailQEntry *ThisItem = vQE;
+				StrBufPrintf(ThisItem->StatusMessage,
+					     "No relay configured matching %s / %s", 
+					     (Author != NULL)? Author : "",
+					     (Address != NULL)? Address : "");
+				ThisItem->Status = 5;
+
+				nActivated++;
+
+				if (i > 1) n = MsgCount++;
+				syslog(LOG_INFO,
+				       "SMTPC: giving up on <%ld> <%s> %d / %d \n",
+				       MyQItem->MessageID,
+				       ChrPtr(ThisItem->Recipient),
+				       i,
+				       m);
+				(*((int*) userdata)) ++;
+				smtp_try_one_queue_entry(MyQItem,
+							 ThisItem,
+							 Msg,
+							 KeepBuffers,
+							 n,
+							 RelayUrls);
+
+				if (KeepBuffers) HaveBuffers++;
+
+				i++;
+			}
+			if (Author != NULL) free (Author);
+			if (Address != NULL) free (Address);
+			DeleteHashPos(&It);
+
+			return;
+		}
+		if (Author != NULL) free (Author);
+		if (Address != NULL) free (Address);
+
 		while ((i <= m) &&
 		       (GetNextHashPos(MyQItem->MailQEntries,
 				       It, &len, &Key, &vQE)))
@@ -963,7 +1050,7 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 							 n,
 							 RelayUrls);
 
-				if (KeepBuffers) HaveBuffers = 1;
+				if (KeepBuffers) HaveBuffers++;
 
 				i++;
 			}
