@@ -94,7 +94,7 @@ const char *smtp_get_Recipients(void)
 
 	if (sSMTP == NULL)
 		return NULL;
-	else return sSMTP->from;
+	else return ChrPtr(sSMTP->from);
 }
 
 
@@ -271,7 +271,8 @@ void smtp_do_bounce(char *instr, StrBuf *OMsgTxt)
 			CtdlOutputMsg(omsgid,
 				      MT_RFC822,
 				      HEADERS_ALL,
-				      0, 1, NULL, 0);
+				      0, 1, NULL, 0,
+				      NULL, NULL);
 
 			StrBufAppendBuf(BounceMB, CC->redirect_buffer, 0);
 			FreeStrBuf(&CC->redirect_buffer);
