@@ -4,21 +4,21 @@
  * http://www.spamassassin.org (the SpamAssassin project is not in any way
  * affiliated with the Citadel project).
  *
- * Copyright (c) 1998-2009 by the citadel.org team
+ * Copyright (c) 1998-2012 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
+ * 
+ * 
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * 
+ * 
+ * 
  */
 
 #define SPAMASSASSIN_PORT       "783"
@@ -104,9 +104,9 @@ int spam_assassin(struct CtdlMessage *msg) {
 		return(0);
 	}
 
-	CCC->sReadBuf = NewStrBuf();
+	CCC->SBuf.Buf = NewStrBuf();
 	CCC->sMigrateBuf = NewStrBuf();
-	CCC->sPos = NULL;
+	CCC->SBuf.ReadWritePointer = NULL;
 
 	/* Command */
 	syslog(LOG_DEBUG, "Transmitting command\n");
@@ -188,7 +188,7 @@ int spam_assassin(struct CtdlMessage *msg) {
 	}
 
 bail:	close(sock);
-	FreeStrBuf(&CCC->sReadBuf);
+	FreeStrBuf(&CCC->SBuf.Buf);
 	FreeStrBuf(&CCC->sMigrateBuf);
 	return(is_spam);
 }
